 /*  -*-objc-*-
 *  ShelfIcon.m: Implementation of the ShelfIcon Class 
 *  of the GNUstep GWorkspace application
 *
 *  Copyright (c) 2001 Enrico Sersale <enrico@imago.ro>
 *  
 *  Author: Enrico Sersale
 *  Date: August 2001
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#include "IconViewsIcon.h"
#include "GWorkspace.h"
#include "Functions.h"
#include "GNUstep.h"

/* File Watcher Strings */
extern NSString *GWFileWatcherFileDidChangeNotification;
extern NSString *GWWatchedDirectoryDeleted;
extern NSString *GWFileDeletedInWatchedDirectory;
extern NSString *GWFileCreatedInWatchedDirectory;

@implementation ShelfIcon

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver: self];
  [self unsetWatcher];
  RELEASE (watched);
  [super dealloc];
}

- (void)setPaths:(NSArray *)fpaths
{
  [self unsetWatcher];
  [super setPaths: fpaths];
  ASSIGN (watched, [[paths objectAtIndex: 0] stringByDeletingLastPathComponent]);
  [self setWatcher];
}

- (void)select
{
	isSelect = YES;
  [namelabel setTextColor: [NSColor blackColor]];
  [self setNeedsDisplay: YES];
}

- (void)unselect
{
	isSelect = NO;
  [namelabel setTextColor: [NSColor blackColor]];
	[self setNeedsDisplay: YES];
}

- (void)setWatcher
{
  [gw addWatcherForPath: watched];

  [[NSNotificationCenter defaultCenter] addObserver: self 
                		selector: @selector(watcherNotification:) 
                				name: GWFileWatcherFileDidChangeNotification
                			object: nil];
}

- (void)unsetWatcher
{
  [[NSNotificationCenter defaultCenter] removeObserver: self];
  [gw removeWatcherForPath: watched];
}

- (void)watcherNotification:(NSNotification *)notification
{
  NSDictionary *notifdict;
  NSMutableArray *names;
  NSArray *newnames;
  BOOL deleted;
  int i;    

  notifdict = (NSDictionary *)[notification object];
  
  if ([[notifdict objectForKey: @"path"] isEqualToString: watched] == NO) {
    return;
  }

  newnames = [fm directoryContentsAtPath: watched];

  if (singlepath == YES) {    
    if ([newnames containsObject: name] == NO) {
      [self unsetWatcher];
      [container removeIcon: self];
    }    
  } else {
  
    names = [NSMutableArray arrayWithCapacity: 1];
    for (i = 0; i < [paths count]; i++) {
      [names addObject: [[paths objectAtIndex: i] lastPathComponent]];
    }  

    deleted = NO;
    for (i = 0; i < [names count]; i++) {
      if ([newnames containsObject: [names objectAtIndex: i]] == NO) {
        [self unsetWatcher];
        [paths removeObjectAtIndex: i];
        deleted = YES;
      }
    }

    if (deleted == YES) {
    
      if ([paths count] == 0) {
        [self unsetWatcher];
        [container removeIcon: self];
        
      } else {
        name = [[NSString alloc] initWithFormat: @"%i items", [paths count]];
        [namelabel setStringValue: cutFileLabelText(name, namelabel, 60)];
        [namelabel setNeedsDisplay: YES];
      }
    }
  }
}

- (void)mouseDown:(NSEvent *)theEvent
{
	NSEvent *nextEvent;
  BOOL startdnd = NO;

	if ([theEvent clickCount] > 1) {  
		unsigned int modifier = [theEvent modifierFlags];
		
		[container openCurrentSelection: paths 
													newViewer: (modifier == NSControlKeyMask)];
    [self unselect];
    return;
	}  
    
  if (isSelect == NO) {  
    [self select];
  }
   
  while (1) {
	  nextEvent = [[self window] nextEventMatchingMask:
    							            NSLeftMouseUpMask | NSLeftMouseDraggedMask];

    if ([nextEvent type] == NSLeftMouseUp) {
      [container setCurrentSelection: paths];
      [self unselect];
      break;

    } else if ([nextEvent type] == NSLeftMouseDragged) {
	    if(dragdelay < 5) {
        dragdelay++;
      } else {        
        startdnd = YES;        
        break;
      }
    }
  }
  
  if (startdnd == YES) {  
    [self startExternalDragOnEvent: nextEvent];    
  }               
}

- (void)startExternalDragOnEvent:(NSEvent *)event
{
	NSEvent *nextEvent;
  NSPoint dragPoint;
  NSPasteboard *pb;

	nextEvent = [[self window] nextEventMatchingMask:
    							NSLeftMouseUpMask | NSLeftMouseDraggedMask];

  if([nextEvent type] != NSLeftMouseDragged) {
    [self unselect];
   	return;
  }
  
  dragPoint = [nextEvent locationInWindow];
  dragPoint = [self convertPoint: dragPoint fromView: nil];

	pb = [NSPasteboard pasteboardWithName: NSDragPboard];	
  [self declareAndSetShapeOnPasteboard: pb];
  	  
  [self dragImage: icon
               at: dragPoint 
           offset: NSZeroSize
            event: event
       pasteboard: pb
           source: self
        slideBack: NO];
}

- (void)draggedImage:(NSImage *)anImage 
						 endedAt:(NSPoint)aPoint 
					 deposited:(BOOL)flag
{
	if (flag == NO) {
    [self unsetWatcher];
    [container removeIcon: self];	
	} else {
	  dragdelay = 0;
	  onSelf = NO;
	  [self unselect];	
	}
}

- (void)declareAndSetShapeOnPasteboard:(NSPasteboard *)pb
{
  NSArray *dndtypes;
  NSData *pbData;

  dndtypes = [NSArray arrayWithObject: NSFilenamesPboardType];
  [pb declareTypes: dndtypes owner: nil];
  pbData = [NSArchiver archivedDataWithRootObject: paths];
  [pb setData: pbData forType: NSFilenamesPboardType];
}

@end
