/*
 *  main.m: main function of GWorkspace.app
 *
 *  Copyright (c) 2000 Enrico Sersale <enrico@imago.ro>
 *  
 *  Author: Enrico Sersale
 *  Date: April 2000
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#include "GWorkspace.h"
#include "Functions.h"
#include "GNUstep.h"

NSFileManager *fmanager = nil;

void createMenu();

int main(int argc, char **argv, char** env)
{
	id pool = [NSAutoreleasePool new];
//	[NSObject enableDoubleReleaseCheck: YES];
	NSApplication *theApp = [NSApplication sharedApplication];
	[theApp setDelegate: [GWorkspace gworkspace]];  
	fmanager = [NSFileManager defaultManager];
	createMenu();
	[theApp run];
	[pool release];
	return 0;
}

void createMenu()
{
	NSMenu *mainmenu;
	NSMenu *info, *file, *edit, *view, *tools;
	NSMenu *inspectors, *fiend;
	NSMenu *windows, *services;  
	NSMenuItem *menuItem;
	
	// Main
   mainmenu = AUTORELEASE ([[NSMenu alloc] initWithTitle: @"GWorkspace"]);
	[[NSApplication sharedApplication] setMainMenu: mainmenu];		
	
	// Info 	
	menuItem = addItemToMenu(mainmenu, @"Info", @"", nil, @"");
	info = AUTORELEASE ([NSMenu new]);
	[mainmenu setSubmenu: info forItem: menuItem];	
	addItemToMenu(info, @"Info Panel...", @"", @"showInfo:", @"");
	addItemToMenu(info, @"Preferences...", @"", @"showPreferences:", @"");
	addItemToMenu(info, @"Help...", @"", nil, @"?");
	 
	// File
	menuItem = addItemToMenu(mainmenu, @"File", @"", nil, @"");
	file = AUTORELEASE ([NSMenu new]);
	[mainmenu setSubmenu: file forItem: menuItem];		
	addItemToMenu(file, @"Open", @"", @"openSelection:", @"o");
	addItemToMenu(file, @"Open as Folder", @"", @"openSelectionAsFolder:", @"O");
	addItemToMenu(file, @"Edit File", @"", @"editFile:", @"e");
	addItemToMenu(file, @"New Folder", @"", @"newFolder:", @"n");
	addItemToMenu(file, @"New File", @"", @"newFile:", @"N");
	addItemToMenu(file, @"Duplicate", @"", @"duplicateFiles:", @"u");
	addItemToMenu(file, @"Destroy", @"", @"deleteFiles:", @"d");
	addItemToMenu(file, @"Empty Recycler", @"", @"emptyRecycler:", @"");
	addItemToMenu(file, @"Put Away", @"", @"putAway:", @"");
	addItemToMenu(file, @"Print...", @"", @"print:", @"p");

	// Edit
	menuItem = addItemToMenu(mainmenu, @"Edit", @"", nil, @"");
	edit = AUTORELEASE ([NSMenu new]);
	[mainmenu setSubmenu: edit forItem: menuItem];	
	addItemToMenu(edit, @"Cut", @"", nil, @"x");
	addItemToMenu(edit, @"Copy", @"", nil, @"c");
	addItemToMenu(edit, @"Paste", @"", nil, @"v");
	addItemToMenu(edit, @"Select All", @"", @"selectAllInViewer:", @"a");

	// View
	menuItem = addItemToMenu(mainmenu, @"View", @"", nil, @"");
	view = AUTORELEASE ([NSMenu new]);
	[mainmenu setSubmenu: view forItem: menuItem];	
				
	// Tools
	menuItem = addItemToMenu(mainmenu, @"Tools", @"", nil, @"");
	tools = AUTORELEASE ([NSMenu new]);
	[mainmenu setSubmenu: tools forItem: menuItem];	
	addItemToMenu(tools, @"Viewer", @"", @"showViewer:", @"V");	
		menuItem = addItemToMenu(tools, @"Inspectors", @"", nil, @"");
		inspectors = AUTORELEASE ([NSMenu new]);
		[tools setSubmenu: inspectors forItem: menuItem];	
		addItemToMenu(inspectors, @"Show Inspectors", @"", nil, @"");
		addItemToMenu(inspectors, @"Attributes", @"", @"showAttributesInspector:", @"1");
		addItemToMenu(inspectors, @"Contents", @"", @"showContentsInspector:", @"2");
		addItemToMenu(inspectors, @"Tools", @"", @"showToolsInspector:", @"3");
		addItemToMenu(inspectors, @"Permissions", @"", @"showPermissionsInspector:", @"4");
	addItemToMenu(tools, @"Finder", @"", @"showFinder:", @"f");
	addItemToMenu(tools, @"Processes...", @"", @"showProsesses:", @"");
		menuItem = addItemToMenu(tools, @"Fiend", @"", nil, @"");
		fiend = AUTORELEASE ([NSMenu new]);
		[tools setSubmenu: fiend forItem: menuItem];	
	addItemToMenu(tools, @"XTerm", @"", @"startXTerm:", @"t");

	// Windows
	menuItem = addItemToMenu(mainmenu, @"Windows", @"", nil, @"");
	windows = AUTORELEASE ([NSMenu new]);
	[mainmenu setSubmenu: windows forItem: menuItem];		
	addItemToMenu(windows, @"Arrange in Front", @"", nil, @"");
	addItemToMenu(windows, @"Miniaturize Window", @"", nil, @"");
	addItemToMenu(windows, @"Close Window", @"", @"closeMainWin:", @"w");
	[[NSApplication sharedApplication] setWindowsMenu: windows];

	// Services 
	menuItem = addItemToMenu(mainmenu, @"Services", @"", nil, @"");
	services = AUTORELEASE ([NSMenu new]);
	[mainmenu setSubmenu: services forItem: menuItem];		
	[[NSApplication sharedApplication] setServicesMenu: services];

	// Hide
	addItemToMenu(mainmenu, @"Hide", @"", @"hide:", @"h");
	
	// Quit
	addItemToMenu(mainmenu, @"Quit", @"", @"terminate:", @"q");

	[mainmenu update];
}

